use std::sync::Arc;
use axum::{
    extract::{Path, State},
};
use axum::Json;
use serde::{Deserialize,Serialize};
use crate::*;
use crate::PowerDnsOidcTsigkeyError;
use url::Url;

pub async fn list_keys(
    State(state):  State<Arc<AppState>>,
) -> Result<Json<Vec<TsigKey>>, PowerDnsOidcTsigkeyError> {
    let req = state.http_client.get::<String>(
            get_url(state.config.powerdns.url.clone(), "localhost".to_owned(), format!("tsigkeys"))
        )
        .header("X-API-Key", state.config.powerdns.api_token.clone());
    let response = req
        .send()
        .await?;
    match response.status().is_success() {
        true => {
            let powerdns_tsig_keys: Vec<PowerDnsTsigKey> = response.json().await?;
            Ok(Json(powerdns_tsig_keys.into_iter().map(|key| key.into()).collect()))
        },
        false => Err(PowerDnsOidcTsigkeyError::from_response(response).await)
    }
}

pub async fn list_key(
    Path(key_id): Path<String>,
    State(state):  State<Arc<AppState>>,
) -> PowerDnsOidcTsigkeyResult<Json<TsigKey>> {
    let key: TsigKey = parse_json::<PowerDnsTsigKey>(
        state.http_client.get::<String>(
            get_url(state.config.powerdns.url.clone(),
                "localhost".to_owned(),
                format!("tsigkeys/{}", key_id))
        )
        .header("X-API-Key", state.config.powerdns.api_token.clone())
        .send()
        .await?
    )
    .await?
    .into();
    Ok(axum::Json(key))
}

pub async fn create_key(
    State(state): State<Arc<AppState>>,
    body: String,
) -> PowerDnsOidcTsigkeyResult<Json<TsigKey>> {
    let key: TsigKey = parse_json::<PowerDnsTsigKey>(
        state.http_client.post(
            get_url(state.config.powerdns.url.clone(), "localhost".to_owned(), "tsigkeys".to_owned())
        )
        .header("X-API-Key", state.config.powerdns.api_token.clone())
        .header("Content-Type", "application/json")
        .body(body)
        .send()
        .await?
    )
    .await?
    .into();
    Ok(axum::Json(key))

}

//pub async fn modify_key(
//    Path(key_id): Path<String>,
//    State(state): State<Arc<AppState>>
//) -> PowerDnsOidcTsigkeyResult<Json<TsigKey>> {}
//
//pub async fn delete_key(
//    Path(key_id): Path<String>,
//    State(state): State<Arc<AppState>>
//) -> PowerDnsOidcTsigkeyResult<Json<Value>> {
//
//}

fn get_url(powerdns_url: Url, server: String, endpoint: String) -> String {
    format!("{}/servers/{}/{}", powerdns_url.to_string(), server, endpoint).as_str().into()
}

#[derive(Serialize, Debug)]
pub struct TsigKeyList {
    pub keys: Vec<TsigKey>,
}

#[derive(Serialize, Debug)]
pub struct TsigKey {
    pub name: String,
    pub algorithm: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub key: Option<String>,
}

#[derive(Deserialize, Serialize, Debug)]
struct PowerDnsTsigKey {
    pub name: String,
    pub id: String,
    pub algorithm: String,
    pub key: String,
    #[serde(rename = "type")]
    pub object_type: String,
}

impl From<PowerDnsTsigKey> for TsigKey {
    fn from(tsigkey: PowerDnsTsigKey) -> Self {
        Self {
            name: tsigkey.name,
            algorithm: tsigkey.algorithm,
            key: Some(tsigkey.key).filter(|s| !s.is_empty())
        }
    }
}

